
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_OPTION_CLASS_H
#define PG_OPTION_CLASS_H

#include "Array.h"
#include "String.h"

class Opt;
class CmdLine;


// a group of command line options
class OptGrp: public Array<Opt*> {
	public:
		virtual ~OptGrp() {}

		virtual ostream &printAnonym(ostream &os) const { return os; }
		virtual bool parseAnonym(const Array<const char *> &) { return true; }
		virtual bool canParseAnonym() const { return false; }

		virtual bool validate() const { return true; }

		void share(OptGrp *other);
};

// a generic option
class Opt {
	public:
		Opt(OptGrp *aGrp, const char *aName, const char *aDescr);
		virtual ~Opt();

		void cmdLine(CmdLine *aCmdLine) { theCmdLine = aCmdLine; }

		virtual bool parse(const char *name, const char *val) = 0;
		virtual void report(ostream &) const = 0;
		virtual bool visible() const { return true; } // use in reports

		const char *name() const { return theName; }
		const char *type() const { return theType; }
		const char *descr() const { return theDescr; }

	protected:
		OptGrp *theGrp;
		CmdLine *theCmdLine;
		String theName;
		String theType;
		String theDescr;
};


#endif
