
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef CLT_XACTION_H
#define CLT_XACTION_H

#include "runtime/httpHdrs.h"
#include "runtime/Xaction.h"

class Client;
class ServerRep;
class DistrPoint;
class WrBuf;

class CltXact: public Xaction {
	public:
		CltXact() { CltXact::reset(); }

		virtual void reset();

		void reqType(int aReqType);

		void enqueue();
		void exec(Client *anOwner, Connection *aConn);

		virtual void noteReadReady(int fd);
		virtual void noteWriteReady(int fd);

		virtual void cacheDistrPoint(DistrPoint *) {}

		bool askedPeer() const { return thePeerState != peerUnknown; }
		bool usePeer() const { return thePeerState == peerSome; }
		void usePeer(bool doUse) { thePeerState = doUse ? peerSome : peerNone; }

	protected:
		virtual Agent *owner();

		void makeReq(WrBuf &buf);
		int interpretHeader();
		bool staleReply() const;
		Size parseContent();
		void parseTag(const char *tagB, const char *tagE);

		virtual void finish(int errNo);
		virtual void logStats(OLog &ol) const;

		virtual void consume(Size size);
		void checkDateSync();

	protected:
		Client *theOwner;
		ServerRep *theSrvRep;
		RepHdr theRepHdr;
		Size theConsumedSize;

		int theOidShortage; // how many new oids we have asked for
		int theFailedOid;   // failed oid to report to the server

		int theReqType;     // IMS/Reload/etc.

		// this is used to check whether the xaction wants to use peers
		enum { peerUnknown = -1, peerNone, peerSome } thePeerState;
};

#endif
