
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "polygraph.h"

#include <iomanip.h>
#include <strstream.h>

#include "StatPhase.h"
#include "StatPhaseMgr.h"
#include "polyBcastChannels.h"

StatPhaseMgr TheStatPhaseMgr;


StatPhaseMgr::StatPhaseMgr(): thePhase(0), thePhaseIdx(-1) {
}

StatPhaseMgr::~StatPhaseMgr() {
	thePhase = 0;
	while (thePhases.count()) delete thePhases.pop();
}

void StatPhaseMgr::addPhase(StatPhase *ph) {
	Assert(ph);
	Assert(!ph->used());
	thePhases.append(ph);
}

void StatPhaseMgr::start() {
	nextPhase();
}

void StatPhaseMgr::forceFlush() {
	for (int i = 0; i < thePhases.count(); ++i)
		if (thePhases[i]->used())
			thePhases[i]->flush();
}

void StatPhaseMgr::noteDone(StatPhase *ph) {
	Assert(thePhase == ph);
	nextPhase();
}

void StatPhaseMgr::nextPhase() {
	const StatPhase *prevPhase = thePhase;

	if (thePhaseIdx >= thePhases.count() - 1) {
		// all phases have finished
		strstream s;
		s << "all " << thePhases.count() << " phases met their goals" << ends;
		Broadcast(ThePhasesEndChannel, s.str());
		s.freeze(false);

		// add a never-finishing phase when all official phases are over
		// so that agents do not have to check for some phase to exist
		// note: this phase is never deleted, but that's OK
		thePhase = new StatPhase;
		thePhase->name("cold");
		thePhase->statsLogged(false);
	} else {
		thePhase = thePhases[++thePhaseIdx];
	}

	Assert(thePhase);
	thePhase->start(this, prevPhase);
}

void StatPhaseMgr::reportCfg(ostream &os) const {

	// header
	os 
		<< setw(10) << "phase"
		<< ' ' << setw(8) << "load_beg"
		<< ' ' << setw(8) << "load_end"
		<< ' ' << setw(8) << " rec_beg"
		<< ' ' << setw(8) << " rec_end"
		<< ' ' << setw(8) << "goal"
		<< "\t flags"
		<< endl;

	for (int i = 0; i < thePhases.count(); ++i)
		thePhases[i]->reportCfg(os);
}
